;
;	>>> ISO-Logo by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	This module requires 11 volatile bytes.
;
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.proc logo

;	Pattern start/end numbers, see scipt
	pattern_logo_start = 4
	pattern_logo_wave = 5
	pattern_logo_rise = 6
	pattern_logo_normal = 8
	pattern_logo_stop = 107

	first_color = $20

	line_height = 11
	line_height_text = 16

;	Non-volatile bytes are used only during the logo intro phase.
;	They are not used/change after the intro phase.
	ypos_start = logo_zp	;1 byte
	ytmp_l1	   = logo_zp+1	;1 byte
	ytmp_l6    = logo_zp+2	;1 byte
	wave_cnt   = logo_zp+3	;2 bytes
	brightness = logo_zp+5	;1 byte

;	Volatile bytes are used to pass on the logo y-positions and colors to the kernel.
	ypos_l0	= volatile_zp	;1 byte
	ypos_l1	= volatile_zp+1	;1 byte
	ypos_l2	= volatile_zp+1	;1 byte, use same value as ypos_l1
	ypos_l3	= volatile_zp+1	;1 byte, use same value as ypos_l1
	ypos_l4	= volatile_zp+1	;1 byte, use same value as ypos_l1
	ypos_l5	= volatile_zp+1	;1 byte, use same value as ypos_l1
	ypos_l6	= volatile_zp+2	;1 byte

	colorfx	= volatile_zp+6 ;5 bytes
 	colufx  = colupf
 
	.macro m_colufx
	lda colorfx+:1+$100		;4 cycles
	.endm


	.proc init
	lda #<system.return
	sta vblank_ptr
	sta playfield_ptr
	lda #>system.return
	sta vblank_ptr+1
	sta playfield_ptr+1
	mva #238-64 wave_cnt
	mva #110 ypos_start
	mva #$50 flash_cnt
	rts
	.endp			;End of init

;===============================================================

	.proc playfield
	lda #$32		;2 copies, medium, missle x8	
	sta nusiz0
	lda #$26		;3 copies, medium, missle x4
	sta nusiz1

	ldx #object.p0		;Position player 0
	lda #74+6
	jsr object.set_hpos			
	lda #57+6		;Position player 1, ldx #object.p1
	jsr object.set_hpos		
	lda #59+6		;Position missle 0, ldx #object.m0
	jsr object.set_hpos	
	lda #46+6		;Position missle 1, ldx #object.m1
	jsr object.set_hpos

	jsr bar.white
	lda #0
	sta wsync
	sta hmove
	sta colubk

	ldx ypos_l0
	bne skip
	ldx #66
	jmp blank

skip	jsr blank

	sta hmclr		;Must be at least 20 cycles after sta hmove
	lda #$11
	sta ctrlpf		;REF = 1 + PLAYER PRIO
	sta hmm0		;Move 1 pixel left on HMOVE
	sta hmp0


	.proc line_top_1_to4
	ldx #3
loop	sta wsync
	sta hmove

	lda #$00
	sta pf1
	lda #$f3
	sta pf2
	lda data.grp1_a+1,x
	sta grp1
	lda #$ff
	sta enam0		;Enable missle 0
	sta grp0		;Endble player 0
	jsr wait.wait16
	lda #$cf
	sta pf2
	lda #$03
	sta pf1
	dex
	bpl loop
	.endp
	assert_same_page line_top_1_to4

	.proc line_top_5
	ldx ypos_l1
	m_colufx 0
	ldy #0
	sta wsync
	sta colupf
	lda #$0f
	sta grp1
	nop
	lda #$35
	sta ctrlpf		;REF = 1 + PF PRIO
	sta nusiz0		;1 copy, double size player 0, missle x8	
	sty enam0		;Disable missle 0
	sty grp0		;Disable player 0
	nop
	nop
	lda #$01
	sta pf1
	lda #$f9
	sta pf2
	nop
	nop
	lda #$9f
	sta pf2
	.endp

	.proc lines_middle_0
;	ldx ypos_l1		;Value already loaded before
	dex
loop	sta wsync
	lda #$f9
	sta enam1		;Disable missle 1
	sta pf2
	m_colufx 0
	sta colufx

	lda #$35		;1 copy, double size player 0, missle x8
	cpx #4			;2
	bcs skip1		;2/3
	ldy data.grp1_b,x	;4
	jmp skip2		;3, could be BCC, beware of page boundary
skip1	ldy #$0f		;2
	sta nusiz0+$100		;4
skip2
	lda #$00
	sta hmm0
	lda #$70
	sta hmp0
	sty grp1		;

	lda #$9f		;End of "S" plus start and middle of "O"
	sta resp0+$100		;Position player in the middle of "S"
	sta pf2
	lda #$0f
	sta enam1		;Enable missile 1
	sta grp1		;Enable player 1
	dex
	bpl loop
	.endp
	assert_same_page lines_middle_0

	.proc lines_middle_1
	ldx ypos_l2
loop	sta wsync
	sta hmove
	lda #$19
	sta enam1		;Disable missle 1
	sta pf2
	m_colufx 1
	sta colufx

	cpx #5			;2
	bcs skip1		;2/3
	ldy data.grp1_a,x	;4
	lda #$ff
	jmp skip2		;3
skip1	ldy #$00		;2
	lda #$00
	stx dummy4		;4
skip2	sta grp0

	and #$10
	sta hmp0

	lda #$0f
	sta enam1		;Enable missle 1 for "S"
	sty grp1		;
	sty dummy3
	sta grp1		;Enable player 1
	sta resm0

	dex
	bpl loop
	.endp
	
	.proc lines_middle_2
	ldx ypos_l3
loop	sta wsync
	lda #$f9
	sta enam1		;Disable missle 1
	sta pf2
	m_colufx 2
	sta colufx
	jsr wait.wait24
	ldy #$0f
	sty grp1

	lda #$99		;End of "S" plus start and middle of "O"
	sta pf2
	lda #$0f
	sta enam1		;Enable missile 1
	sta grp1		;Enable player 1
	dex
	bpl loop
	.endp

	lda #$30
	sta hmp0
	.proc lines_middle_3
	ldx ypos_l4
loop	sta wsync
	sta hmove
	lda #$81
	sta enam1		;Disable missle 1
	sta pf2
	m_colufx 3
	sta colufx

	cpx #5			;2
	bcs skip1		;2/3
	lda #$ff		;2
	.byte $2c		;3
skip1	lda #$00		;2
	sta grp0
	sta enam0
	and #$10
	sta hmp0
	sta hmm0

	lda #$99		;End of "S" plus start and middle of "O"
	sta pf2
	lda #$0f
	sta enam1		;Enable missile 1
	sta grp1		;Enable player 1
	dex
	bpl loop
	.endp

	.proc lines_middle_4
	ldx ypos_l5
	ldy #0
loop	sta wsync
	sta hmove
	lda #$f9
	sta enam1		;Disable missle 1
	sta pf2

	sty enam1		;Disable missle 1
	sty grp0		;Disable player 0
	sty enam0		;Disable missle 0

	m_colufx 4
	sta colufx

	cpx #5			;2
	bcs skip1		;2/3
	lda #$10		;2
	.byte $2c		;3
skip1	lda #$00		;2
	sta hmp1
	dec cxclr		;5

	lda #$9f		;End of "S" plus start and middle of "O"
	sta pf2
	sta enam1		;Enable missile 1
	dex
	bpl loop

	ldx ypos_l6		;Fall through to blank
	bne do_blank
	jmp screen_off.without_sync

do_blank
	jsr screen_off.without_sync
	.endp			;End of lines_middle_4


	.proc blank
loop	sta wsync
	dex
	bne loop
skip	rts
	.endp
	
	.endp			;End of playfield

;===============================================================

	.local data
;pf2_s	.byte $f9,$19,$f9,$81,$f9
;pf2_o	.byte $9f,$19,$99,$99,$9f
grp1_a	.byte $0f,$07,$03,$01	;Used for top, next $00 is used also in case of grp1_a+1
grp1_b	.byte $00,$08,$0c,$0e
	.endl			;End of data


;===============================================================

	.proc vblank

	lda #line_height-1	;Default line height
	sta ypos_l1

	lda #0			;Zero indicates logo is inactive
	sta ypos_l0
	sta ypos_l6

	ldx sound.pattern_hi
	cpx #pattern_logo_start	;No logo before this pattern...
	bcc set_colors		;<A>=0, i.e. no flashing
	cpx #pattern_logo_stop	;No logo after this pattern...
	bcc do_logo		;<A>=0, i.e. no flashing
	rts

do_logo	cpx #pattern_logo_normal;Normal logo after this pattern...
	bcc do_animate

	lda #2			;Activate logo with default spacing
	sta ypos_l0
	lda sound.pattern_hi
	and #$f0
	adc #2+first_color
	sta colorfx+4
	adc #2
	tax			;Main color 1 (colup1)
	adc #2
	sta colorfx+3
	adc #2
	sta colorfx+2
	adc #2
	sta colorfx+1
	adc #2
	tay			;Main color 2 (colupf, colup1)
	adc #2
	sta colorfx+0
	bne set_colors.set_main_colors

do_animate			;Animate logo between pattern ... and ...
	cpx #pattern_logo_rise
	bcc do_not_rise
	lda ypos_start
	beq do_not_rise
	dec ypos_start
do_not_rise

	lda #line_height_text	;Use starting height from text

	cpx #pattern_logo_wave	;Start waving at pattern ..
	bcc do_not_wave

	lda wave_cnt
	beq wave_zero
	dec wave_cnt
wave_zero
;	clc
	adc #5			;C==1; ensure final height equals "line_height"
	jsr eor_63
	adc #5
	cmp #32
	bcc *+4
	lda #31
do_not_wave
	sta ypos_l1		;Set a line height
	asl			;multiply by 5
	asl
;	clc			;C==0
	adc ypos_l1		;0...79
	lsr
	sta ytmp_l1		;floor(y/2)
	adc #0
	sta ytmp_l6		;ceil(y/2)

	lda ypos_start		;Compute upper padding
;	clc			;C==0
	adc #30
;	sec			;C==0
	sbc ytmp_l1
	sta ypos_l0

	lda #190		;Compute lower padding
;	sec			;C==1
	sbc ytmp_l6
	sbc ypos_start
	sta ypos_l6

	dec ypos_l1		;Correct line height value for playfield kernel
	
	lda flash_cnt		;Fall through to animate colors
	
	.proc set_colors
	clc
	adc #16
	jsr eor_63
	sbc #13			;C==0
	sta brightness

	.proc compute_colors
	jsr add_brightness
	sta colorfx+4
	jsr add_brightness
	tax			;Main color 1 (colup1)
	jsr add_brightness
	sta colorfx+3
	jsr add_brightness
	sta colorfx+2
	jsr add_brightness
	sta colorfx+1
	jsr add_brightness
	tay			;Main color 2 (colupf, colup1)
	jsr add_brightness
	sta colorfx+0
	.endp			;End of compute colors

	lda flash_cnt
	cmp #$30		;Borders are black when fading down first
	bcc set_main_colors
	ldx #0
	ldy #0
set_main_colors
	stx colup1		;Main color 1
	sty colupf		;Main color 2
	sty colup0
	rts

	.proc add_brightness
	lda brightness
	bpl positive
	lda #0
	beq black
positive
	cmp #16
	bcc small
	lda #15
small	ora #first_color
black	inc brightness
	inc brightness
	rts
	.endp			;End of add_brightness
	.endp			;End of set_colors
	.endp			;End of vblank

;	IN: <A>=0..255
;	OUT: <A>=0..31, <C>=0
	.proc eor_63
	and #63
	cmp #32
	bcc *+4
	eor #63
	clc
	rts
	.endp

	.endp			;End of logo

	.print "PROC  logo:	", logo, " - ", logo+.len logo-1, " (", .len logo, " bytes)"
	.print "LOCAL logo.data:	", logo.data, " - ", logo.data+.len logo.data-1, " (", .len logo.data, " bytes)"
