;---------------------------------------------------------------------------
;
; 13 Char Text demo
;
; by Paul Slocum, Andrew Towers, Thomas Jentzsch
;
;---------------------------------------------------------------------------


	icl "../VCS.asm"

	opt h-l+

;---------------------------------------------------------------------------
; Constants
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; RAM Variables
;---------------------------------------------------------------------------

	org $80
chara	.ds 5	;Must be at $80 because stakc point is checked using BMI
charb	.ds 5
charc	.ds 5
chard	.ds 5
chare	.ds 5
charf	.ds 5
charg	.ds 5
chars		;End of character buffer+1

colors	.ds 5

temp 	.ds 1
line 	.ds 1
count	.ds 1
textptr	.ds 1
savesp	.ds 1

	.macro m_textdisp
	lda charg+:1	;3
	sta enam0	;3
	lsr		;2
	sta enam1	;3
	lsr		;2
	sta enabl	;3
	lda charf+:1	;3
	sta grp0	;3
	lda chare+:1	;3
	sta.w grp1	;4
	ldx.w colors+:1	;2
	nop		;2
	stx colup0	;3
	stx colup1	;3
	lda chard+:1	;3
	sta grp0	;3
	lda charc+:1	;3
	ldy charb+:1	;3
	stx colupf	;3
	ldx chara+:1	;3
	sta grp1	;3
	sty grp0	;3
	stx grp1	;3
	sta grp0	;3
	lda #$20	;2
	sta colupf	;3
	.endm

	opt h-f+
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
	org $F000
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; Start of Program
;---------------------------------------------------------------------------
; Clear memory, locate character graphics positions and such,
; initialize key memory values, start GameLoop.
;-------------------------------------------------------------
start
	sei
	cld
	ldx #$FF
	txs
	lda #0
clear
	sta 0,x
	dex
	bne clear

;---------------------------------------------------------------------------
; Initialize variables / registers
;---------------------------------------------------------------------------

	lda #$48
	sta COLUP0
	lda #$99
	sta COLUP1
	lda #255
	sta PF0
	sta PF1
	lda #0
	sta PF2

	lda #%00000101
	sta WSYNC
	sta CTRLPF
	sta GRP0
	sta GRP1
	lda #$03		; set both players to 3 copies
	sta NUSIZ0
	sta NUSIZ1
	lda #$01		; set vertical delay on for both players
	sta VDELP0
	sta $2d	; nop
	sta RESM0
	sta RESM1
	sta VDELP1
	sta RESBL
	sta RESP0
	sta RESP1
	lda #%11110000
	sta HMP1
	lda #%11100000
	sta HMP0
	lda #%10000000
	sta HMBL
	lda #%10110000
	sta HMM0
	lda #%01010000
	sta HMM1
	sta WSYNC
	sta HMOVE


;--------------------------------------------------------------------------
; GameLoop
;--------------------------------------------------------------------------
	.proc GameLoop
	jsr vsync_area	;start vertical retrace

	jsr vblank_area	; spare time during screen blank
	jsr playfield_area	; draw one screen
	jsr overscan_area	; do overscan

	jmp GameLoop	;back to top
	.endp

;--------------------------------------------------------------------------
; VSync
;--------------------------------------------------------------------------
	.proc vsync_area
	lda #2	  ;bit 1 needs to be 1 to start retrace
	sta VSYNC	;start retrace
	sta WSYNC	;wait a few lines
	sta WSYNC
	lda #44	 ;prepare timer to exit blank period (44)
	sta TIM64T	;turn it on
	sta WSYNC	;wait one more
	sta VSYNC	;done with retrace, write 0 to bit 1

	rts
	.endp		;End of vsync_area



;--------------------------------------------------------------------------
; VBlank
;--------------------------------------------------------------------------
; Game Logic
;--------------------------------------------------------------------------
	.proc vblank_area

	rts
	.endp		;End of vblank_area


;--------------------------------------------------------------------------
; Overscan
;--------------------------------------------------------------------------
; More Game Logic
;--------------------------------------------------------------------------
	.proc overscan_area

	sta WSYNC

	lda #$00
	sta COLUBK

	lda #36	 ; Use the timer to make sure overscan takes (34)
	sta TIM64T	; 30 scan lines.  29 scan lines * 76 = 2204 / 64 = 34.4

sync	lda INTIM	; We finished, but wait for timer
	bne sync	; by looping till zero

	sta WSYNC	; End last scanline

	lda #$82
	sta VBLANK
	lda #$02
	sta VBLANK

	rts
	.endp		;End of overscan_area



;--------------------------------------------------------------------------
; Draw TV Pictures
;--------------------------------------------------------------------------
;
;--------------------------------------------------------------------------


	.proc playfield_area

pictureLoop
	lda INTIM	;check timer for end of VBLANK period
	bne pictureloop ;loop until it reaches 0

	sta wsync
	lda #$80
	sta vblank	;end screen blank

	; Set playfield color
	lda #$20
	sta colupf

	lda #15
	sta line

	lda #0
	sta textptr

	tsx
	stx savesp

	sta wsync
	jsr wait.wait24
	nop
	nop
	nop
	nop
	jmp mainTextLoop	;3

; put this here so we can jump out of the main text loop
; using a 2-cycle relative branch.
quitTextLoop
	lda #0
	sta grp0
	sta grp1
	sta grp0
	sta grp1
	sta enam0
	sta enam1
	sta enabl

; restore stack pointer
	ldx savesp
	txs

	ldx #11
ScanLoop
	sta wsync
	dex
	bne scanloop

	sta colupf

	rts	; Picture

;------------------------------------------------------------------

	.proc wait		;Delay a defined number of cycles
wait24	nop
wait22	nop
wait20	nop
wait18	nop
wait16	nop
wait14	nop
wait12
return	rts
	.endp			;End of wait

	.align 256

	.proc mainTextLoop

	lda textptr
	asl
	asl
	asl
	asl
;	clc
	adc #2
	sta colors+0
	adc #2
	sta colors+1
	adc #2
	sta colors+2
	adc #2
	sta colors+3
	adc #2
	sta colors+4


; set up the stack for building screen data

	ldx #chars-1	;2
	txs		;2

; load the first text character
; note: x and y are reversed!
	ldy textptr	;3
	ldx text1,y	;4
	beq quitTextLoop ;2 ->4	; look for terminating character (0)
	iny		;2

; build screen data for first character

	lda fontlo,x	;4
	pha		;3
	lda fontlo+1,x	;4
	pha		;3
	lda fontlo+2,x	;4
	pha		;3
	lda fontlo+3,x	;4
	pha		;3
	lda fontlo+4,x	;4
	pha		;3

; loop through the remaining 6 character pairs
textLineLoop

; load the next two text characters

	ldx text1,y	;4
	lda text2,y	;4
	iny		;2
	sty textptr	;3
	tay		;2

; build screen data for next two characters

	lda fonthi,x	;4
	ora fontlo,y	;4
	pha		;3
	lda fonthi+1,x	;4
	ora fontlo+1,y	;4
	pha		;3
	lda fonthi+2,x	;4
	ora fontlo+2,y	;4
	pha		;3
	lda fonthi+3,x	;4
	ora fontlo+3,y	;4
	pha		;3
	lda fonthi+4,x	;4
	ora fontlo+4,y	;4
	pha		;3

; next character pair
	ldy textptr	;3
	tsx		;2
	bmi textLineLoop ;2

; now render it asap
	nop

	m_textdisp 0
	m_textdisp 1
	m_textdisp 2
	m_textdisp 3
	m_textdisp 4

	lda #0		;2
	sta colup0	;3
	sta colup1	;3
	sta.w enabl	;4	use up one extra cycle left over!

	jmp mainTextLoop	;3

	.endp

	.endp		;End of Picture
;---------------------------------------------------------------------------
; note: dasm doesn't like using x or y as a label!

	.align 256
	.local text1
	.byte <fonthi.t  ,<fonthi.h  ,<fonthi.r  ,<fonthi.e  ,<fonthi.n  ,<fonthi.h  ,<fonthi.r
	.byte <fonthi.sp ,<fonthi.sp ,<fonthi.sp ,<fonthi.e  ,<fonthi.o  ,<fonthi.ast,<fonthi.sp
	.byte <fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq
	.byte <fonthi.t  ,<fonthi.h  ,<fonthi.s  ,<fonthi.i  ,<fonthi.sp ,<fonthi.sp ,<fonthi.sp
	.byte <fonthi.w  ,<fonthi.h  ,<fonthi.l  ,<fonthi.sp ,<fonthi.u  ,<fonthi.c  ,<fonthi.sp
	.byte <fonthi.o  ,<fonthi.f  ,<fonthi.s  ,<fonthi.u  ,<fonthi.f  ,<fonthi.i  ,<fonthi.a
	.byte <fonthi.w  ,<fonthi.r  ,<fonthi.t  ,<fonthi.n  ,<fonthi.sp ,<fonthi.o  ,<fonthi.sp
	.byte <fonthi.t  ,<fonthi.e  ,<fonthi.t  ,<fonthi.t  ,<fonthi.e  ,<fonthi.n  ,<fonthi.w
	.byte <fonthi.t  ,<fonthi.e  ,<fonthi.t  ,<fonthi.r  ,<fonthi.u  ,<fonthi.i  ,<fonthi.e
	.byte <fonthi.i  ,<fonthi.s  ,<fonthi.i  ,<fonthi.sp ,<fonthi.o  ,<fonthi.sp ,<fonthi.sp
	.byte <fonthi.ast,<fonthi.ast,<fonthi.e  ,<fonthi.yy ,<fonthi.c  ,<fonthi.o  ,<fonthi.ast
	.byte <fonthi.yy ,<fonthi.e  ,<fonthi.sp ,<fonthi.t  ,<fonthi.i  ,<fonthi.sp ,<fonthi.sp
	.byte <fonthi.a  ,<fonthi.c  ,<fonthi.u  ,<fonthi.l  ,<fonthi.yy ,<fonthi.sp ,<fonthi.sp
	.byte <fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq
	.byte <fonthi.o  ,<fonthi.n  ,<fonthi.sp ,<fonthi.o  ,<fonthi.e  ,<fonthi.l  ,<fonthi.n
	.byte 0 ; terminate text1                                             
	.endl

	.local text2
	.byte 0  ,<fonthi.i  ,<fonthi.t  ,<fonthi.e  ,<fonthi.c  ,<fonthi.a  ,<fonthi.s
	.byte 0  ,<fonthi.ast,<fonthi.d  ,<fonthi.m  ,<fonthi.sp ,<fonthi.sp ,<fonthi.sp
	.byte 0  ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq
	.byte 0  ,<fonthi.i  ,<fonthi.sp ,<fonthi.s  ,<fonthi.a  ,<fonthi.sp ,<fonthi.sp
	.byte 0  ,<fonthi.o  ,<fonthi.e  ,<fonthi.b  ,<fonthi.n  ,<fonthi.h  ,<fonthi.sp
	.byte 0  ,<fonthi.sp ,<fonthi.t  ,<fonthi.f  ,<fonthi.sp ,<fonthi.sp ,<fonthi.m
	.byte 0  ,<fonthi.i  ,<fonthi.i  ,<fonthi.g  ,<fonthi.t  ,<fonthi.sp ,<fonthi.sp
	.byte 0  ,<fonthi.s  ,<fonthi.sp ,<fonthi.h  ,<fonthi.sp ,<fonthi.e  ,<fonthi.sp
	.byte 0  ,<fonthi.xx ,<fonthi.sp ,<fonthi.o  ,<fonthi.t  ,<fonthi.n  ,<fonthi.sp
	.byte 0  ,<fonthi.sp ,<fonthi.t  ,<fonthi.n  ,<fonthi.t  ,<fonthi.sp ,<fonthi.sp
	.byte 0  ,<fonthi.v  ,<fonthi.r  ,<fonthi.sp ,<fonthi.o  ,<fonthi.l  ,<fonthi.ast
	.byte 0  ,<fonthi.s  ,<fonthi.i  ,<fonthi.sp ,<fonthi.s  ,<fonthi.sp ,<fonthi.sp
	.byte 0  ,<fonthi.t  ,<fonthi.a  ,<fonthi.l  ,<fonthi.sp ,<fonthi.sp ,<fonthi.sp
	.byte 0  ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq ,<fonthi.eq
	.byte 0  ,<fonthi.e  ,<fonthi.m  ,<fonthi.r  ,<fonthi.sp ,<fonthi.i  ,<fonthi.e
	.endl         


;---------------------------------------------------------------------------

	.align 256
	.local fonthi

eof
	.byte %00000000  ; value not used, address must be zero
sp
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
a
	.byte %10100000
	.byte %10100000
	.byte %11100000
	.byte %10100000
	.byte %01000000
b
	.byte %11000000
	.byte %10100000
	.byte %11000000
	.byte %10100000
	.byte %11000000
c
	.byte %01100000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %01100000
d
	.byte %11000000
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %11000000
e
	.byte %11100000
	.byte %10000000
	.byte %11000000
	.byte %10000000
	.byte %11100000
f
	.byte %10000000
	.byte %10000000
	.byte %11000000
	.byte %10000000
	.byte %11100000
g
	.byte %01100000
	.byte %10100000
	.byte %10100000
	.byte %10000000
	.byte %01100000
h
	.byte %10100000
	.byte %10100000
	.byte %11100000
	.byte %10100000
	.byte %10100000
i
	.byte %01000000
	.byte %01000000
	.byte %01000000
	.byte %01000000
	.byte %01000000
j
	.byte %10000000
	.byte %01000000
	.byte %01000000
	.byte %01000000
	.byte %11100000
k
	.byte %10100000
	.byte %10100000
	.byte %11000000
	.byte %10100000
	.byte %10000000
l
	.byte %11100000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
m
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %11100000
	.byte %10100000
n
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %11000000
o
	.byte %01000000
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %01000000
p
	.byte %10000000
	.byte %10000000
	.byte %11000000
	.byte %10100000
	.byte %11000000
q
	.byte %01100000
	.byte %11100000
	.byte %10100000
	.byte %10100000
	.byte %01000000
r
	.byte %10100000
	.byte %10100000
	.byte %11000000
	.byte %10100000
	.byte %11000000
s
	.byte %11000000
	.byte %00100000
	.byte %01000000
	.byte %10000000
	.byte %01100000
t
	.byte %01000000
	.byte %01000000
	.byte %01000000
	.byte %01000000
	.byte %11100000
u
	.byte %01100000
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %10100000
v
	.byte %01000000
	.byte %01000000
	.byte %10100000
	.byte %10100000
	.byte %10100000
w
	.byte %10100000
	.byte %11100000
	.byte %10100000
	.byte %10100000
	.byte %10100000
xx
	.byte %10100000
	.byte %10100000
	.byte %01000000
	.byte %10100000
	.byte %10100000
yy
	.byte %01000000
	.byte %01000000
	.byte %01000000
	.byte %10100000
	.byte %10100000
z
	.byte %11100000
	.byte %10000000
	.byte %01000000
	.byte %00100000
	.byte %11100000
ast
	.byte %10100000
	.byte %01000000
	.byte %11100000
	.byte %01000000
	.byte %10100000
eq
	.byte %00000000
	.byte %11100000
	.byte %00000000
	.byte %11100000
	.byte %00000000
	.endl

	.align 256
	.local fontlo
;eof
	.byte %00000000
;sp
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
;a
	.byte %00001010
	.byte %00001010
	.byte %00001110
	.byte %00001010
	.byte %00000100
;b
	.byte %00001100
	.byte %00001010
	.byte %00001100
	.byte %00001010
	.byte %00001100
;c
	.byte %00000110
	.byte %00001000
	.byte %00001000
	.byte %00001000
	.byte %00000110
;d
	.byte %00001100
	.byte %00001010
	.byte %00001010
	.byte %00001010
	.byte %00001100
;e
	.byte %00001110
	.byte %00001000
	.byte %00001100
	.byte %00001000
	.byte %00001110
;f
	.byte %00001000
	.byte %00001000
	.byte %00001100
	.byte %00001000
	.byte %00001110
;g
	.byte %00000110
	.byte %00001010
	.byte %00001010
	.byte %00001000
	.byte %00000110
;h
	.byte %00001010
	.byte %00001010
	.byte %00001110
	.byte %00001010
	.byte %00001010
;i
	.byte %00000100
	.byte %00000100
	.byte %00000100
	.byte %00000100
	.byte %00000100
;j
	.byte %00001000
	.byte %00000100
	.byte %00000100
	.byte %00000100
	.byte %00001110
;k
	.byte %00001010
	.byte %00001010
	.byte %00001100
	.byte %00001010
	.byte %00001000
;l
	.byte %00001110
	.byte %00001000
	.byte %00001000
	.byte %00001000
	.byte %00001000
;m
	.byte %00001010
	.byte %00001010
	.byte %00001010
	.byte %00001110
	.byte %00001010
;n
	.byte %00001010
	.byte %00001010
	.byte %00001010
	.byte %00001010
	.byte %00001100
;o
	.byte %00000100
	.byte %00001010
	.byte %00001010
	.byte %00001010
	.byte %00000100
;p
	.byte %00001000
	.byte %00001000
	.byte %00001100
	.byte %00001010
	.byte %00001100
;q
	.byte %00000110
	.byte %00001110
	.byte %00001010
	.byte %00001010
	.byte %00000100
;r
	.byte %00001010
	.byte %00001010
	.byte %00001100
	.byte %00001010
	.byte %00001100
;s
	.byte %00001100
	.byte %00000010
	.byte %00000100
	.byte %00001000
	.byte %00000110
;t
	.byte %00000100
	.byte %00000100
	.byte %00000100
	.byte %00000100
	.byte %00001110
;u
	.byte %00000110
	.byte %00001010
	.byte %00001010
	.byte %00001010
	.byte %00001010
;v
	.byte %00000100
	.byte %00000100
	.byte %00001010
	.byte %00001010
	.byte %00001010
;w
	.byte %00001010
	.byte %00001110
	.byte %00001010
	.byte %00001010
	.byte %00001010
;xx
	.byte %00001010
	.byte %00001010
	.byte %00000100
	.byte %00001010
	.byte %00001010
;yy
	.byte %00000100
	.byte %00000100
	.byte %00000100
	.byte %00001010
	.byte %00001010
;z
	.byte %00001110
	.byte %00001000
	.byte %00000100
	.byte %00000010
	.byte %00001110
;ast
	.byte %00001010
	.byte %00000100
	.byte %00001110
	.byte %00000100
	.byte %00001010
;eq
	.byte %00000000
	.byte %00001110
	.byte %00000000
	.byte %00001110
	.byte %00000000
	.endl


;---------------------------------------------------------------------------
; Program Startup
;---------------------------------------------------------------------------
	org $fffc
	.word start
	.word start
