
;	>>> ISO-Scroll by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	This module requires 2 volatile bytes.
;
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.proc scroll

columns		= 8		;Number of colunmns (bytes)
rows		= 7		;Number of rows (block lines), last line is unused/empty
picture_bpl	= 64		;Bytes per line in the picture
black_columns	= 11		;Number of leading and trailing black column

ysize_max	= 21		;Number of scalines per row, actual value is +1
ysize_height	= rows*(ysize_max+1)+2		
border_color	= 14		;Color for upper and lower border of scroll

mode	= scroll_zp+0		;Solid, Flash, Wave, Wave and Scale
xpos	= scroll_zp+1		;X position
xsoft	= scroll_zp+2		;Number of cycles to delay, computed during vblank, used during playfield, 1 byte
ysize	= scroll_zp+3		;Y start position, 1 byte
ypad1	= scroll_zp+4		;Upper padding height, 1 byte
ypad2	= scroll_zp+5		;Lower padding height, 1 byte

sm_bpc	= rows			;SM bytes per column
sm_size	= columns*rows		;SM total size
sm	= ypad2+1		;sm_size bytes
sm_end	= sm+sm_size

scroll_zp_end = sm_end

	.print "ZP scroll_sm:	", sm, " - ", sm_end-1, " (", sm_end-sm, " bytes)"

.if scroll_zp_end > zp_end
	.error "ZP scroll_zp:	", scroll_zp, " - ", scroll_zp_end-1, " (", scroll_zp_end-scroll_zp, " bytes)"
.else
	.print "ZP scroll_zp:	", scroll_zp, " - ", scroll_zp_end-1, " (", scroll_zp_end-scroll_zp, " bytes)"
.endif

;===============================================================

	.proc init
	m_set_effect vblank playfield

	lda #0
	ldx #scroll_zp_end-scroll_zp
clear	sta scroll_zp,x
	dex
	bpl clear
	lda #$d0	;TODO
	sta cnt			;To make sure that bouncing starts at the correct time

	lda #$f0
	sta flash_cnt
	rts
	.endp			;End of init

;===============================================================

	.proc vblank
flash_and = volatile_zp+0
flash_idx = volatile_zp+1

	.proc animate_mode
	ldx #0
	stx flash_and
	stx flash_idx

	lda mode
	beq phase1
	cmp #1
	beq phase2
	cmp #2
	beq phase3
	ldx #3			;Once 3 is reached, stay at 3
	stx mode	
phase3
	dec flash_and

phase2	lda cnt
	lsr
	sta flash_idx
phase1
	.endp			;End of animate_mode

	.proc animate_size
	txa			;<X>==0 means mode is not 3
	beq do_max
	lda cnt
	bmi do_max
	clc
	adc #ysize_max

	and #63
	cmp #31
	bcc *+4
	eor #63

	cmp #ysize_max
	bcc no_max
do_max	lda #ysize_max
no_max	sta ysize

	lda #ysize_height
	ldx #rows
loop	clc
	sbc ysize
	dex
	bne loop
	lsr
	sta ypad1
	adc #0
	sta ypad2
	.endp			;End of animate_size

	.proc animate_scroll
	ldx xsoft		;Soft scrolling and hard scrolling
	ldy xpos
	inx			;Next xsoft
	cpx #7
	bne no_xsoft
	ldx #0
	iny			;Next xpos
	cpy #picture_bpl+black_columns
	bne no_xpos
	ldy #0
	inc mode		;Next mode
no_xpos
no_xsoft
	stx xsoft
	sty xpos

	ldx #0
copy_loop
	lda picture+picture_bpl*0-black_columns,y
	jsr animate_cell
	sta sm+0,x
	lda picture+picture_bpl*1-black_columns,y
	jsr animate_cell
	sta sm+1,x
	lda picture+picture_bpl*2-black_columns,y
	jsr animate_cell
	sta sm+2,x
	lda picture+picture_bpl*3-black_columns,y
	jsr animate_cell
	sta sm+3,x
	lda picture+picture_bpl*4-black_columns,y
	jsr animate_cell
	sta sm+4,x
	lda picture+picture_bpl*5-black_columns,y
	jsr animate_cell
	sta sm+5,x
	lda picture+picture_bpl*6-black_columns,y
	jsr animate_cell
	sta sm+6,x
	iny
	txa
	clc
	adc #rows
	tax
	cpx #rows*columns
	bne copy_loop

is_black
	lda #0			;Moved here to save one RTS
	rts

	.proc animate_cell
	cpy #picture_bpl+black_columns
	bcs is_black
	cpy #black_columns
	bcc is_black
	cmp #32
	bcs normal
	and flash_and
	adc flash_idx
	and #31
	cmp #16
	bcc *+4
	eor #31
normal	rts
	.endp			;End of animate_cell

	.endp			;End of animate_scroll

	.endp			;End of vblank

;===============================================================
	
	.proc playfield

	sta wsync
	sta wsync
	lda #14
	sta wsync
	sta colubk
	ldx #1
	stx ctrlpf

	.proc padding_1		;Upper padding
loop	sta wsync
	lda coltab-1,x
	sta colubk
	lda #$10		;7*(7*21)+3=150 pixel always there, hence 2*4=8 of 10 pixel to mask left and right
	sta pf0			;If 2 remaining pixel appear on the right and are not really visible
	inx
	dec ypad1
	bne loop
	.endp

	sta wsync
	lda flash_cnt
	sta colubk

	sta wsync
	lda xsoft
	beq *+2		;2/3
	cmp #2
	bcc *+2
	cmp #3
	bcc *+2
	cmp #4
	bcc *+2
	cmp #5
	bcc *+2
	cmp #6
	bcc *+2

	jsr wait.wait20
	jsr wait.wait18

	ldy ysize
	iny
	ldx #0

	.local loop
	dey
	bpl skip1	;3+3 = 2+2+2 = 6
	inx		;2
	ldy ysize	;3
	bne skip2	;3

skip1	pha		;3
	pla		;4
skip2
	cpx #rows
	bcs exit

	lda sm+rows*0,x	;4
	sta colubk+$100	;3 = 7
	lda sm+rows*1,x
	sta colubk
	lda sm+rows*2,x
	sta colubk
	lda sm+rows*3,x
	sta colubk
	lda sm+rows*4,x
	sta colubk
	lda sm+rows*5,x
	sta colubk
	lda sm+rows*6,x
	sta colubk
	lda sm+rows*7,x
	sta colubk
	bcc loop

	.endl			;End of loop

	assert_same_page loop

exit	lda flash_cnt
	sta colubk
	sta wsync

	.proc padding_2		;Lower padding
	ldy ypad2
	bne start
loop	sta wsync
	lda coltab+$50,y
	sta colubk
start	dey
	bne loop		;End of padding_2
	.endp
	rts

	.endp			;End of playfield

;===============================================================

	.endp			;End of scroll

	.print "PROC scroll:	", scroll, " - ", scroll+.len scroll-1, " (", .len scroll, " bytes)"

