;
;	>>> ISO-Sound by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	This replayer is based on SONGMAIN.BIN by Qotile/Paul Slocum.
;	This module requires 4 volatile bytes.
;
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.proc sound

pattern_delay	= sound_zp		;0..2
pattern_lo	= sound_zp+1		;0..31
pattern_hi	= sound_zp+2		;0..14, total length = 3*32*15*8 = 11520 NTSC frames = 192s = 3m 12s

voice_number	= volatile_zp		;volatile, 1 byte
pattern_ptr	= volatile_zp+1		;volatile, 2 bytes
temp		= volatile_zp+3		;volatile, 1 byte

;===============================================================
;	Player to be called every frame

	.proc player			;Play two voices
	ldy pattern_hi
	lda #0
	ldx tune.track1,y
	bpl not_last_pattern
	sta pattern_hi
	lda #0
	ldx tune.track1,y
not_last_pattern
;	lda #0
	jsr play_voice
	lda pattern_hi
	clc
	adc #.len tune.track1
	tay
	ldx tune.track1,y
	lda #1
	jsr play_voice

	ldy #0				;Increament delay, lo and hi counters
	dec pattern_delay
	bpl same_pattern_lo

	lda #tune.delay-1
	sta pattern_delay

	ldx pattern_lo
	inx
	stx pattern_lo
	
	cpx #32
	bne same_pattern_hi
	inc pattern_hi
	sty pattern_lo
same_pattern_hi
same_pattern_lo
	rts


;===============================================================
;	Play one voice
;	IN: <A>=voice #0 or #1
;	IN: <X>=tune.track... 
	.proc play_voice
	sta voice_number

	lda tune.pattern.addresses,x
	sta pattern_ptr
	lda tune.pattern.addresses+1,x
	sta pattern_ptr+1
	ldy pattern_lo
	lda (pattern_ptr),y
	beq silence

;	Get frequency and distortion from packed pattern byte
;	IN:  <A> = pattern byte, %DDDFFFFF with %DDD = 3 bit distortion control index, %FFFFF = 5 bit frequency or $ff
;	OUT: <A> = audf0/1 value
;	OUT: <X> = audc/1 value
	.proc get_audfc
	cmp #$ff
	beq exit_ff
	tax
	and #$1F
;	pha
	sta temp
	txa
	lsr
	lsr
	lsr
	lsr
	lsr
	tax
	lda tune.control,x
	tax
;	pla
	lda temp
;	.byte $2c	;Skip next 2 bytes
exit_ff
;	lda #$ff
	.endp		;End of get_audfc

	ldy voice_number
	sta audf0,y
	stx audc0,y
	cmp #$ff
	beq exit_ff

	.proc get_audv
	lda pattern_lo
	and #$18
	lsr
	lsr
	lsr
	ora #$20
	tay
	lda pattern_lo
	and #7
	tax
	lda (pattern_ptr),y
	and bitmask,x
	beq bit_zero
	lda #$0F
	.byte $2c	;Skip next 2 bytes
bit_zero
	lda #$06
	.endp		;End of get_audv

silence
	ldy voice_number
	sta audv0,y

;--------------------------------------------------------------------------
; Super High Hat (TM)
;--------------------------------------------------------------------------
; This plays the high hat sound on the first frame of each beat indicated
; in hatPattern
;--------------------------------------------------------------------------
	ldy voice_number
	beq noHat

	; Read high hat pattern
	lda pattern_hi
	cmp #tune.hat_start
	bmi noHat

	lda pattern_lo
	and #%00000001
	;cmp #%00000001
	;tax
	;lda beat
	;lsr
	;lsr
	;lsr
	;tay
	;lda hatPattern,y
	;and bitMaskArray,x
	;beq noHat
	beq noHat

	; Only play hat on first frame
	lda pattern_delay
	bne noHat

	; Play hat
	lda #tune.hat_pitch
	sta audf1
	lda #tune.hat_sound
	sta audc1
	lda #tune.hat_volume
	sta audv1
noHat

exit_ff	rts


bitmask	.byte $80,$40,$20,$10,$08,$04,$02,$01	;Used by get_audv

	.endp
	.endp


	.proc tune
	icl "ISO-Sound-Tune-Pattern.asm"
	icl "ISO-Sound-Tune-Tracks.asm"
	.endp

	.print "PROC tune.track1:", tune.track1, " - ", tune.track1+.len tune.track1-1, " (", .len tune.track1, " bytes)"
	.print "PROC tune.track2:", tune.track2, " - ", tune.track2+.len tune.track2-1, " (", .len tune.track2, " bytes)"

	.endp

	.print "PROC sound:", sound, " - ", sound+.len sound-1, " (", .len sound, " bytes)"
	.print "PROC sound.tune:", sound.tune, " - ", sound.tune+.len sound.tune-1, " (", .len sound.tune, " bytes)"
	.print "PROC sound.tune.pattern:", sound.tune.pattern, " - ", sound.tune.pattern+.len sound.tune.pattern-1, " (", .len sound.tune.pattern, " bytes)"

