
;	>>> ISO-Text by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	40x10 pixel of text screen, 3 objects moving in 160x60 pixels (double scanlines).
;
;	See http://www.randomterrain.com/atari-2600-memories-tutorial-andrew-davie-17.html
;	for the exact playfield timings regarding splits of PF0/1/2.
;
;	Define the following global state variables in the including file
;	text_ptr	.ds 2
;	text_cnt	.ds 1
;	Define text_zp and zp_end in the including file ($68 bytes required).
;	This module requires 4 volatile bytes.
;	The first 20 bytes after text_zp are also volatile.
;
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.proc text

; Block mode (fading or falling), also used as bit pattern for NUSIZx and as object width/height
	.enum mode
	stopped = $00
	fading	= $01
	falling	= $fe
	.ende

; Block states, positions and speeds
	.enum state
	free	 = $00
	starting = $30
	moving	 = $3f
	wide	 = $10	;Bit 4 indicates the width of the block
	.ende

; Block operations, the upper 4 bits contain the first/implicit operand <#>
	.enum operation
	call	= 0	;<<pc_lo,>pch>
	wait	= 1	;<,steps>
	position= 2	;<#,x,y+[$80]>
	move	= 3	;<#,direction,steps>
	fall	= 4	;<>
	.ende

; Block move directions
	.enum direction
	up	= 0
	down	= 1
	left	= 2
	right	= 3
	.ende
	
;===============================================================

; Persistent zero page definitions

; Text main area - players and missles
text_pm_size	  = (text_sm_ppl*text_sm_lines)/2	;20 bytes, bits (6,5,4) and (3,2,1) used
text_pm		  = text_zp				;Volatile, may be overwritten during VBLANK

; Text lower area - players and missles
text_pm_lower_start  = 40+1-block_size_falling			;
text_pm_lower_height = 20
text_pm_lower_limit  = 38


; Text main area - playfield
text_sm_ppl	  = 4				;4 pixels per line, 2 scanlines per pixel
text_sm_lines	  = 10				;10 pixels high
text_sm_bpl	  = 6				;40 pixels, 6 bytes per line
text_sm_size	  = text_sm_bpl*text_sm_lines	;60 bytes
text_sm		  = text_pm+text_pm_size


block_mode	= text_sm+text_sm_size
block_pixel_x	= block_mode+1
block_pixel_y	= block_pixel_x+1

block_count	   = 3		;Number of blocks
block_size_fading  = 8		;Size of blocks
block_size_falling = 8		;Size of blocks
block_max_x	   = 159-8	;Assume 2x2 block as maximum
block_max_y	   = text_sm_lines*4+20-1
block_x_speed	   = 2		;X speed
block_y_gravity	   = $c0	;Gravity which accelates the Y speed

block_state	 = block_pixel_y+1
block_x		 = block_state+block_count
block_y		 = block_x+block_count
block_y_lo	 = block_y+block_count
block_vx	 = block_y_lo+block_count
block_vy	 = block_vx+block_count
block_vy_lo	 = block_vy+block_count
block_move_count = block_y_lo

; Ensure that the zero page address do not overwrite the call stack
text_zp_end = block_vy_lo+block_count


.if text_zp_end > zp_end
	.error "ZP text_zp:	", text_zp, " - ", text_zp_end-1, " (", text_zp_end-text_zp, " bytes)"
.else
	.print "ZP text_zp:	", text_zp, " - ", text_zp_end-1, " (", text_zp_end-text_zp, " bytes)"
.endif

; Volatile zero page definition
pixel_x = volatile_zp+0
pixel_y = volatile_zp+1
pixel_offset = volatile_zp+2
pixel_bit =  volatile_zp+3
pixel_offset2 = volatile_zp+4
pixel_bit2    = volatile_zp+5

;===============================================================

	.proc data
pm_offsets	.byte 3,3,2,2,1,1;Overlaps with tab_vx (2 bytes)
move_tab_vx	.byte  0,0	 ;Overlaps with tab_vy (2 bytes)
move_tab_vy	.byte -1,1,0	 ;Overlaps with pixel_mix (1 byte)
pixel_mix	.byte 0,6,4,8,2

;===============================================================

	.local script

	.macro m_call
	.byte operation.call
	.word :1
	.endm

	;If the argument is >=$80, it is assumed to be a memory location which must become zero or $ff to continue
	.macro m_wait
	.byte operation.wait,:1
	.endm

	.macro m_wait_sound
	.byte operation.wait,[:1 & $3f] | $40
	.endm

	.macro m_position_at
	.byte operation.position+:1*$10,:2,:3*2+$00
	.endm

	.macro m_start_at
	.byte operation.position+:1*$10,:2,:3*2+$01
	.endm
	
	.macro m_move
	.if :0=3
	.byte operation.move+:1*$10,:2,[:3/4]*$10
	.else
	.byte operation.move+:1*$10,:2,[:3/4]*$10+[:4/4]
	.endif
	.endm

	.macro m_fall
	.byte operation.fall
	.endm
	
	icl "ISO-Text-Script.asm"
	
	 .endl			;End of script
	
	.print "PROC script:	", script, " - ", script+.len script-1, " (", .len script, " bytes)"
	.print "LOCAL script.iso:	", .len script.iso, " bytes"
	.print "LOCAL script.ribbon:	", .len script.ribbon, " bytes"
	.print "LOCAL script.plasma:	", .len script.plasma, " bytes"
	.print "LOCAL script.twist:	", .len script.twist, " bytes"
	.print "LOCAL script.twirl:	", .len script.twirl, " bytes"
	.print "LOCAL script.scroll:	", .len script.scroll, " bytes"

	.endp			;End of data

;===============================================================

	.proc init

	.if >playfield.without_bar <> >playfield
	.error "playfield.without_bar=",playfield.without_bar," and playfield=",playfield, " have different page"
	.endif



	mwa #data.script text_ptr

;	TODO: Uncomment for script testing
;	mwa #data.script.plasma text_ptr
;	mva #data.script.plasma.start sound.pattern_hi
;	mwa #data.script.twist text_ptr
;	mva #data.script.twist.start sound.pattern_hi
;	mwa #data.script.twirl text_ptr
;	mva #data.script.twirl.start sound.pattern_hi
;	mwa #data.script.scroll text_ptr
;	mva #data.script.scroll.start sound.pattern_hi
;	mwa #data.script.finish text_ptr
;	mva #data.script.finish.start sound.pattern_hi
;	mva #1 text_stage

last	lda #<playfield.without_bar
	.byte $2c
next	lda #<playfield
	sta playfield_ptr
	lda #>playfield
	sta playfield_ptr+1
	mwa #vblank vblank_ptr

	lda #0
	sta text_cnt
	ldx #text_zp_end-text_zp+1
clear	sta text_zp-1,x
	dex
	bne clear
;	mva #mode.fading block_mode
	inx			;Save 1 byte
	stx block_mode
	rts
	.endp			;End of init


	.proc animate

;===============================================================

	ldx text_cnt
	beq do_next
	bmi check_memory	;>=$80 means check memory for $00 or $ff
	cpx #$40		;Between $40 and $7f mean wait for sound pattern matching
	bcs check_sound
	dec text_cnt
	beq do_next
return	rts

check_sound
	.if .def text_test	;In text test mode, there is no sound sync
	jmp do_next
	.else
	txa
	eor sound.pattern_hi
	and #$3f		;sound.pattern_hi & $3f = text_cnt & $3f ?
	beq do_next
	rts
  	.endif
  
check_memory
	lda $00,x		;Use as text_cnt as memory location
	beq do_next
	cmp #$ff
	bne return

do_next	jsr get_byte		;Get next byte
	tay
	lsr			;Move upper nibble to <X>
	lsr
	lsr
	lsr
	tax
	tya			;Move lower nibble to <Y>
	and #$0f
	tay
	beq do_call
	dey
	beq do_wait
	dey
	beq do_position
	dey
	beq do_move

	.proc do_fall
	mva #mode.falling block_mode
	mva #block_mode text_cnt	;Wait until block_mode becomes mode.stopped==$00

	lda #39
	sta block_pixel_x
	lda #0
	sta block_pixel_y

;	lda #state.free		;A==0
	sta block_state
	sta block_state+1
	sta block_state+2
	rts
	.endp			;End of do_fall

	.proc do_call
	jsr get_byte
	sta animate_ptr
	jsr get_byte
	sta animate_ptr+1
	jmp call_animate	;Called routine must end with RTS
	.endp
	
	.proc do_wait
	jsr get_byte
	sta text_cnt
	rts			;Return
	.endp

	.proc do_move
	jsr get_byte
	tay
	lda data.move_tab_vx,y
	sta block_vx,x
	lda data.move_tab_vy,y
	sta block_vy,x
	lda #state.moving
	sta block_state,x

	jsr get_byte
	tay			;The upper 4 bits contain the number of move blocks
	lsr			;Multiply by 4
	lsr
	and #$3c
	sta block_move_count,x
	tya			;The lower 4 bits contain the number of wait blocks
	asl			;Multiply by 4
	asl
	and #$3c
	beq do_next
	sta text_cnt
	rts
	.endp			;End of do_move

	.proc do_position
	jsr get_byte
	sta block_x,x
	jsr get_byte
	lsr
	sta block_y,x
	bcc no_start
	lda #state.starting
	sta block_state,x
no_start
	sty block_vx,x		;Y==0
	sty block_vy,x
	jmp do_next
	.endp


	.proc get_byte
	ldy #0
	lda (text_ptr),y
	inw text_ptr
	rts
	.endp			;End of get_byte
	
	.endp			;End of animate
;===============================================================

;	IN: <X> not used, returned unchanged
;	IN: pixel_x,pixel_y
;	OUT: <A>=pixel_bits = bit mask with 1 bit set, <Y>=buffer offset
	
	.proc locate_pixel	;Converts (pixel_x,pixel_y) to buffer offset and bit mask
	ldy #0
	lda pixel_x
	cmp #20
	bcc is_left
	sbc #20
	ldy #3
is_left	pha
	cmp #4
	bcc is_left_4
	cmp #12
	bcc is_left_12
	iny
is_left_12
	iny
is_left_4
	sty pixel_offset
	pla
	tay
	lda pixel_bits,y
	sta pixel_bit

	ldy pixel_y
	lda pixel_lines,y
	clc
	adc pixel_offset
	tay
	lda pixel_bit
	rts

pixel_bits	.byte $10,$20,$40,$80,$80,$40,$20,$10,$08,$04,$02,$01,$01,$02,$04,$08,$10,$20,$40,$80
pixel_lines	.byte 0*text_sm_bpl,1*text_sm_bpl,2*text_sm_bpl,3*text_sm_bpl,4*text_sm_bpl
		.byte 5*text_sm_bpl,6*text_sm_bpl,7*text_sm_bpl,8*text_sm_bpl,9*text_sm_bpl
	.endp			;End of locate_pixel


;===============================================================
;	IN: <X> not used, returned unchanged
;	IN: pixel_x,pixel_y

	.proc plot_pixel	;Plots one pixel at (pixel_x,pixel_y)

	jsr locate_pixel
	ora text_sm,y
	sta text_sm,y
return	rts
	.endp			;End of plot_pixel

;===============================================================

	.proc vblank

	ldx #block_count-1
	
	.proc move_blocks	;Move blocks and plot_pixels
loop	lda block_state,x	;Block in use at all?
	beq next_block

	bit block_mode		;Which mode
	bmi falling

	.proc fading
	cmp #state.moving	;Fade from "state.starting" to "state.moving"
	bcs already_moving
	inc block_state,x
already_moving

	lda block_move_count,x	;Block still moving?
	beq is_not_moving
	dec block_move_count,x

	clc
	lda block_x,x
	adc block_vx,x
	sta block_x,x

	clc
	lda block_y,x
	adc block_vy,x
	sta block_y,x
is_not_moving

	lda block_y,x		;Y must not be larger then the max
	cmp #text_sm_lines*text_sm_ppl
	bcs next_block
	ora block_x,x		;X|Y must be 0 MOD 3
	and #3
	bne next_block

	lda block_x,x
	lsr
	lsr
	sta pixel_x
	lda block_y,x
	lsr
	lsr
	sta pixel_y

	jsr plot_pixel		;Plot a 2x2 pixel
	inc pixel_x
	jsr plot_pixel
	inc pixel_y
	jsr plot_pixel
	dec pixel_x
	jsr plot_pixel
	bne next_block
	.endp			;End of fading

	.proc falling
	clc
	lda block_x,x
	adc block_vx,x
	cmp #block_max_x	;<0 also means >block_max_x
	bcs toggle_x
	sta block_x,x
	bcc not_toggle_x
toggle_x
	lda block_vx,x
	eor #$ff
	adc #0
	sta block_vx,x
not_toggle_x

;	clc			;Ignore lowest bit addition
	lda block_y_lo,x	;16 bit y positioning
	adc block_vy_lo,x
	sta block_y_lo,x
	lda block_y,x
	adc block_vy,x
	sta block_y,x

;	clc			;C==0 because Y is only moving down
	lda block_vy_lo,x	;Increase Y speed
	adc #block_y_gravity
	sta block_vy_lo,x
	bcc skip
	inc block_vy,x
skip
	.endp			;End of falling

next_block
	dex
	bpl loop	
	.endp			;End of move_blocks

	.proc falling		;Falling
	lda block_mode
	cmp #mode.falling
	bne return

	ldx #block_count-1
loop	lda block_state,x	;Block free?
	bne is_moving		;No
	lda block_pixel_x	;Anything left on the screen
	bmi next		;No
	sta pixel_x
	ldy block_pixel_y
	lda data.pixel_mix,y
	sta pixel_y
	jsr locate_pixel
	sta pixel_bit2
	sty pixel_offset2

	and text_sm,y
	beq no_pixel_found	;If the first pixel is not set, the 2nd doesn't matter

	dec pixel_x
;;	bmi screen_empty	;Does not happen
	jsr locate_pixel
	and text_sm,y
	bne pixel_found_2x2
	beq pixel_found_1x1

no_pixel_found
	inc block_pixel_y
	ldy block_pixel_y
	cpy #text_sm_lines/2	;Blocks are always 2 lines high
	bcc next		;Not find next to prevent stall
	sta block_pixel_y	;A==0 because of "and text_sm,y"
	dec block_pixel_x
	rts

pixel_found_2x2
	jsr eor_pixel
	lda #state.moving	;$30 = Width x8
	.byte $2c
pixel_found_1x1
	lda #state.moving-$10	;$20 = Width x4
	sta block_state,x

	ldy pixel_offset2
	lda pixel_bit2
	jsr eor_pixel

	lda pixel_x
	asl
	asl
	sta block_x,x
	lda pixel_y
	asl
	asl
	sta block_y,x

	inc pixel_x		;Back to original x position

	lda #block_x_speed
	sta block_vx,x
	lda #1
	sta block_vy,x		;Randomization by not setting "block_vy_lo"
	bne next

is_moving
	lda block_y,x
	cmp #block_max_y+2
	bcc no_max
	lda #state.free
	sta block_state,x
no_max

next	dex
	bpl loop

	lda block_pixel_x	;Any pixel left on the screen?
	bpl are_moving
	lda block_state		;Any block still active?
	ora block_state+1
	ora block_state+2
	bne are_moving
	sta block_mode		;A==0, equals mva #mode.stopped block_mode
are_moving
return	rts

	.proc eor_pixel
	pha
	eor text_sm,y		;Clear pixel
	sta text_sm,y
	pla
	eor text_sm+6,y		;Clear pixel in next line
	sta text_sm+6,y
	rts
	.endp

	.endp			;End of falling

	.endp			;End of vblank

;===============================================================		
	
	.proc playfield

lower_enam0 = volatile_zp+0
lower_enam1 = volatile_zp+1
lower_enabl = volatile_zp+2

	sta wsync
	sta wsync

	jsr bar.white
	lda #0
	jsr bar.store

without_bar
	lda #44
	sta tim64t

	jsr plot_pm
	
;	Set x positions for M0, M1 and BL
	ldx #object.m0
hpos_loop
	lda block_x-object.m0,x
	clc
	adc #16
	jsr object.set_hpos
	cpx #object.bl+1
	bcc hpos_loop
	jsr object.move_hpos
	
	ldx #0			;Set colors for M0,M1
	jsr set_block_color
	inx
	jsr set_block_color

	lda #14			;Set colors for BL and PFx
	sta colupf
	
	ldy #text_pm_lower_start;Preset shadow registers for first line
	jsr set_lower_enablings

	lda block_state+0	;Set block sizes
	and #$30
	sta nusiz0
	lda block_state+1
	and #$30
	sta nusiz1
	lda block_state+2
	and #$30
	sta ctrlpf		;No scope, no reflect

loop	lda intim		;Sync with beam again
;	sta colubk
	bne loop
	sta wsync

	.macro m_text_sm_line	;Draws one SM line with height text_sm_ppl*2
	ldy #text_sm_ppl*2-1

loop	ldx data.pm_offsets,y
	.if :1 < text_sm_lines/2;Use lower bits 4,3,2 for 1st half of rows
	lda text_pm+text_sm_ppl*:1,x	
	.else			;Use upper bits 7,6,5 for 2nd half of rows
	lda text_pm+text_sm_ppl*(:1-text_sm_lines/2),x
	lsr
	lsr
	lsr
	.endif

	ldx text_sm+text_sm_bpl*:1
	sta wsync
	stx pf0

	sta enam0		;Bit 2 = MISSLE 0 enable
	lsr			;Next bit
	sta enam1		;Bit 3 = MISSLE 1 enable
	lsr			;Next bit
	sta enabl		;Bit 4 = BALL enable

	ldx text_sm+1+text_sm_bpl*:1
	stx pf1
	ldx text_sm+2+text_sm_bpl*:1
	stx pf2
	nop

	ldx text_sm+3+text_sm_bpl*:1
	stx pf0
	ldx text_sm+4+text_sm_bpl*:1
	stx pf1
	ldx text_sm+5+text_sm_bpl*:1
	stx pf2

	dey
	bpl loop
	.endm

;	Unrolled loop
	.rept text_sm_lines
	m_text_sm_line #
	.endr

	ldy #text_pm_lower_start+1
	pha				;Delay 4+3 cycles
	pla
	lda #0
	sta pf0
	sta pf1
	sta pf2

	.local lower_loop
	sta wsync
	mva lower_enam0 enam0	
	mva lower_enam1 enam1
	mva lower_enabl enabl
	ldx #2				;Set color for BL
	jsr set_block_color

	sta wsync
	jsr set_lower_enablings		;Compute next line
	iny
	cpy #text_pm_lower_start+text_pm_lower_height+1
	bne lower_loop
	.endl
	rts

;===============================================================

	.proc plot_pm	;Plots all blocks into the text_pm memory
	pm_height_count = volatile_zp+0
	pm_bits	= locate_pixel.pixel_bits+13	;Bits $02,$04,$08 for M0,M1,BL

	ldx #text_pm_size-1
	lda #0
clear	sta text_pm,x
	dex
	bpl clear

	ldx #block_count-1
block_loop
	lda block_state,x
	beq skip_block
	ldy #block_size_fading-1
	bit block_mode
	bpl fading
	ldy #block_size_falling-1
fading	sty pm_height_count
line_loop
	lda block_y,x
	clc
	adc pm_height_count
	cmp #text_sm_lines*text_sm_ppl
	bcs skip_line
	cmp #(text_sm_lines*text_sm_ppl)/2
	bcc lower1		;Was it in the lower area?
	sbc #(text_sm_lines*text_sm_ppl)/2
lower1	tay			;Use as offset
	lda pm_bits,x	;Contains 
	bcc lower2		;Was it in the lower area?
	asl			;Shift 3 bits right
	asl
	asl
lower2	ora text_pm,y		;Mask bit in
	sta text_pm,y
skip_line
	dec pm_height_count	;Next line
	bpl line_loop
skip_block
	dex			;Next object
	bpl block_loop
	rts
	.endp			;End of plot_pm

;===============================================================

	.proc set_block_color		;Compute fading color
	lda block_y,x
	sec
	sbc #text_pm_lower_limit
	bcs skip1
	lda block_state,x
	and #$0f
	bcc skip2
skip1	lsr
	eor #15
skip2	sta colup0,x
	rts
	.endp

;===============================================================
;	IN: <Y>= scan line between text_pm_lower_start and text_pm_lower_start+text_pm_lower_height+1
	.proc set_lower_enablings	;Preset shadow registers
	tya			;2, for M0
	sbc block_y+0		;3
	adc #block_size_falling	;3
	lda #1			;2
	adc #0			;2
	sta lower_enam0		;3 = 17 cycles, C:=0

	tya			;2, for M1
	sbc block_y+1		;3
	adc #block_size_falling	;3
	lda #1			;2
	adc #0			;2
	sta lower_enam1		;3 = 15 cycles, C:=0

	tya			;2, for BALL
	sbc block_y+2		;3
	adc #block_size_falling	;3
	lda #1			;2
	adc #0			;2
	sta lower_enabl		;3 = 15 cycles
	rts
	.endp			;End of set_lower_enablings

	
	.endp			;End of playfield

;===============================================================

	.endp			;End of text
