;
;	>>> ISO-Twister by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	This module requires 4 volatile bytes.
;
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.proc twister


twister_lines	= 79;		;Double scan lines
twister_mode	= twister_zp
twister_colubk	= twister_zp+1
twister_cnt	= twister_zp+2	;2 bytes
twister_pattern	= twister_zp+4	;1 byte
twister_tab	= twister_zp+6	;twister_lines bytes

; Fields initialized during init
eor_color1	= twister_tab+twister_lines ;1 byte
eor_color2	= eor_color1+1	;1 byte
eor_color3	= eor_color2+1	;1 byte

; Constants
xpos_offset	= 27		;Maximum offset for p0 and p1, so no wsync overrun occurs
xpos_bl_offset	= 23

; Fields prepared during vblank
xpos		= eor_color3+1	;1 byte, only during initalization
xpos_p0		= xpos+1	;1 byte
xpos_p1		= xpos_p0+1	;1 byte
xpos_bl		= xpos_p1+1	;1 byte

lastp0		= xpos_bl+1	;1 byte
lastp1		= lastp0+1	;1 byte
lastbl		= lastp1+1	;1 byte

next_colup0	= lastbl+1	;1 byte
next_colup1	= next_colup0+1	;1 byte
next_grp0	= next_colup1+1	;1 byte

twister_zp_end	= next_grp0+1

.if twister_zp_end > zp_end
	.error "ZP twister_zp:	", twister_zp, " - ", twister_zp_end-1, " (", twister_zp_end-twister_zp, " bytes)"
.else
	.print "ZP twister_zp:	", twister_zp, " - ", twister_zp_end-1, " (", twister_zp_end-twister_zp, " bytes)"
.endif

;===============================================================

	.proc data

	.local twister_sinp0
	.byte $06,$08,$0a,$0c,$0a,$0c,$0e,$0c,$0e,$0c,$0e,$0c,$0e,$0c,$0a,$08
	.endl

	.local twister_grp0
	.byte $ff,$ff,$ff,$ff,$7f,$7f,$7f,$3f,$3f,$1f,$1f,$0f,$0f,$07,$03,$01
	.endl

	.local twister_sinp1
	.byte $06,$05,$04,$03,$02,$01,$00,$00,$00,$00,$00,$01,$02,$03,$04,$05
	.endl

	.local twister_sinbl
	.byte $27,$28,$29,$2a,$2b,$2c,$2d,$2d,$2d,$2d,$2d,$2c,$2b,$2a,$29,$28
	.endl

	.endp

;	Asserts are not required because the loop has uo to 8 cycles tolarance
;	assert_same_page data.twister_sinp0
;	assert_same_page data.twister_grp0
;	assert_same_page data.twister_sinp1
;	assert_same_page data.twister_sinbl
	

;===============================================================

	.proc init
	m_set_effect vblank playfield
	lda #0
	ldx #(twister_zp_end-twister_zp)
clear	sta twister_zp,x
	dex
	bpl clear

next	ldx twister_mode
	inc twister_mode
	lda color1,x
	sta eor_color2
	eor color2,x
	sta eor_color3
	eor #$0f
	sta eor_color1
	lda #$4f		;Fade in
	.byte $2c
fade	lda #-$4f		;Fade out
	jmp flash_fade.with

color1	.byte $0f,$4f,$6f
color2	.byte $00,$20,$70
	.endp			;End of init

;===============================================================

	.proc vblank
	direction = volatile_zp+0;1 byte
	counter = volatile_zp+1	;2 bytes
	speed = volatile_zp+3	;1  byte

	.proc flash
	jsr flash.get_color_and_pattern
	bit flash_cnt
	bmi negative
	eor #$0f
negative
	sta twister_colubk
	stx twister_pattern
	.endp

	.proc animate
	lda twister_cnt+1
	and #1
	beq *+4
	lda #$ff
	sta direction

	lda twister_cnt+1	;Change speed every 5 seconds
	lsr
	lsr

	lda twister_cnt		;$00...$7f...$00
	sta counter+1
	bpl *+4
	eor #$ff

	bcc no_asl		;Speed flag C set?
	asl
no_asl	eor direction
	sta speed
	tax			;Arbitray initialization

	ldy #twister_lines-1
loop
	txa
	adc speed		;C=0 from ADC or CPY before
	tax
	lda direction
	adc counter+1		;C=1 if overflow from counter
	sta counter+1

	and #$1f		;32 phases
	cmp #$10		;Move bit 4 to bit 0
	rol
	and #$1f
	sta twister_tab,y
	dey
	bpl loop
	
	inw twister_cnt
	.endp			;End of animate

	.proc prepare

	ldx twister_mode	;No swing in mode 1 and 2
	cpx #3
	bcc no_swing
	lda twister_cnt
	asl
	tax
no_swing
	lda sinus256,x
	lsr
	lsr
	clc
	adc #xpos_offset
	sta xpos

	lda twister_tab+twister_lines-1	;Y between $70 and $8f
	lsr
	tay
	lda data.twister_sinp0,y
	sta lastp0
	clc
	adc xpos
	sta xpos_p0
	lda data.twister_sinp1,y
	sta lastp1
	clc
	adc xpos
	sta xpos_p1
	lda data.twister_sinbl,y
	sta lastbl
	clc
	adc xpos
	sec
	sbc #xpos_bl_offset
	sta xpos_bl

	lda twister_tab+twister_lines-1;Precompute first line
	lsr
	tay
	bcs skip
	eor eor_color1		;Swap brightness and color of p0/p1
skip	eor eor_color2
	sta next_colup0
	eor eor_color3
	sta next_colup1
	lda data.twister_grp0,y
	sta next_grp0
	.endp			;End of prepare

	rts
	.endp			;End of vblank

;===============================================================
	
	.proc playfield
	ldx #object.p0
	lda xpos_p0
	jsr object.set_hpos
	lda xpos_p1
	jsr object.set_hpos

	lda #14
	sta wsync
	sta colubk
	lda #7
	sta nusiz0		;1 copy, size x 4
	sta nusiz1
	lda #0
	sta wsync
	sta colubk
	sta colupf
	lda #$30
	sta pf0
	lda #$35		;Ball 8x, PF priority
	sta ctrlpf

	ldy twister_colubk
	lda twister_pattern
	bne is_active
	sta wsync
	sty colubk
	ldx #twister_lines*2-1
	bne blank

is_active
	sta wsync
	sty colubk
	lda xpos_bl
	sec
	.proc set_twister_hpos	;Inlined for speed
	sbc #$0f		;2 => 5*3 pixel = 15 pixel per interation 
	bcs set_twister_hpos	;2 => 5 cycles per iteration, branch must not cross page-boundary
	.endp
	assert_same_page set_twister_hpos

	eor #$07		;2
	asl			;2
	asl			;2
	asl			;2
	asl			;2
	sta hmbl		;4 => Fine positioning value
	sta resbl		;3 => Coarse positioning

	lda #$ff
	sta enabl
	sta grp1

	ldx #twister_lines-2	;Loop con

	.local loop
	sta wsync		;Bit 2 must be set
	sta hmove
	lda next_grp0
	sta grp0
	lda next_colup0
	sta colup0
	lda next_colup1
	sta colup1

	lda twister_tab,x
	lsr
	tay
	bcs skip
	eor eor_color1		;Swap brightness and color of p0/p1
skip	eor eor_color2
	sta next_colup0
	eor eor_color3
	sta next_colup1
	
	lda data.twister_grp0,y
	sta next_grp0

	sec
	lda lastp0
	sbc data.twister_sinp0,y
	asl
	asl
	asl
	asl
	sta hmp0

	sec
	lda lastp1
	sbc data.twister_sinp1,y
	asl
	asl
	asl
	asl
	sta hmp1
	
	sec
	lda lastbl
	sbc data.twister_sinbl,y
	asl
	asl
	asl
	asl
	sta hmbl

	lda data.twister_sinp1,y
	sta lastp1
	lda data.twister_sinp0,y
	sta lastp0
	lda data.twister_sinbl,y
	sta lastbl
	dex
	bpl loop
	.endl loop

	ldx #1
blank	sta wsync
	lda #0
	sta grp0
	sta grp1
	sta enabl
	dex
	bne blank
	assert_same_page loop

	rts
	.endp			;End of playfield

;===============================================================

	.endp			;End of twister

	.print "PROC twister:	", twister, " - ", twister+.len twister-1, " (", .len twister, " bytes)"

